@@ -2,6 +2,7 @@
  *   Copyright (C) 2007 by Tobias Koenig <tokoe@kde.org>                   *
  *   Copyright (C) 2008 by Marco Martin <notmart@gmail.com>                *
  *   Copyright (C) 2008-2011 Matthias Fuchs <mat69@gmx.net>                *
+ *   Copyright (C) 2012 Reza Fatahilah Shah <rshah0385@kireihana.com>      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -24,8 +25,6 @@
 #include "comicarchivejob.h"
 #include "checknewstrips.h"
 #include "stripselector.h"
-#include "comicwidgets.h"
-#include "buttonbar.h"
 #include "comicsaver.h"
 
 #include <QtCore/QTimer>
@@ -35,6 +34,8 @@
 #include <QtGui/QGraphicsSceneMouseEvent>
 #include <QtGui/QGraphicsSceneWheelEvent>
 #include <QtGui/QSortFilterProxyModel>
+#include <QtDeclarative/QDeclarativeEngine>
+#include <QtDeclarative/QDeclarativeContext>
 
 #include <KAction>
 #include <KConfigDialog>
@@ -45,13 +46,11 @@
 #include <KStandardShortcut>
 
 #include <Plasma/Containment>
-#include <plasma/tooltipmanager.h>
+#include <Plasma/DeclarativeWidget>
+#include <Plasma/Package>
 
-#include "arrowwidget.h"
 #include "comicmodel.h"
 #include "configwidget.h"
-#include "fullviewwidget.h"
-#include "imagewidget.h"
 
 K_GLOBAL_STATIC( ComicUpdater, globalComicUpdater )
 
@@ -60,8 +59,6 @@ const int ComicApplet::CACHE_LIMIT = 20;
 ComicApplet::ComicApplet( QObject *parent, const QVariantList &args )
     : Plasma::PopupApplet( parent, args ),
       mDifferentComic( true ),
-      mShowPreviousButton( false ),
-      mShowNextButton( false ),
       mShowComicUrl( false ),
       mShowComicAuthor( false ),
       mShowComicTitle( false ),
@@ -70,37 +67,36 @@ ComicApplet::ComicApplet( QObject *parent, const QVariantList &args )
       mArrowsOnHover( true ),
       mMiddleClick( true ),
       mCheckNewStrips( 0 ),
-      mMainWidget( 0 ),
-      mCentralLayout( 0 ),
-      mBottomLayout( 0 ),
-      mFullViewWidget( 0 ),
+      mDeclarativeWidget( 0 ),
       mActionShop( 0 ),
       mEngine( 0 ),
-      mTabAdded( false ),
-      mButtonBar(0),
-      mSavingDir(0)
+      mSavingDir(0),
+      mActiveComicModel(parent)
 {
     setHasConfigurationInterface( true );
     resize( 600, 250 );
     setAspectRatioMode( Plasma::IgnoreAspectRatio );
 
     setPopupIcon( "face-smile-big" );
-
-    graphicsWidget();
 }
 
 void ComicApplet::init()
 {
-    connect(this, SIGNAL(appletTransformedByUser()), this, SLOT(slotSizeChanged()));
-
-    Plasma::ToolTipManager::self()->registerWidget( this );
-
     globalComicUpdater->init( globalConfig() );
     mSavingDir = new SavingDir(config());
 
     configChanged();
-    
-    buttonBar();
+    //QML
+    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(this);
+    mDeclarativeWidget = new Plasma::DeclarativeWidget(this);
+    layout->addItem(mDeclarativeWidget);
+
+    mDeclarativeWidget->engine()->rootContext()->setContextProperty("comicApplet", this);
+
+    Plasma::PackageStructure::Ptr structure = Plasma::PackageStructure::load("Plasma/Generic");
+    Plasma::Package package(QString(), "org.kde.comic", structure);
+    mDeclarativeWidget->setQmlPath(package.filePath("mainscript"));
+    //End of QML
     
     mEngine = dataEngine( "comic" );
     mModel = new ComicModel( mEngine->query( "providers" ), mTabIdentifier, this );
@@ -131,7 +127,7 @@ void ComicApplet::init()
     mActionNextNewStripTab->setShortcut( KStandardShortcut::openNew() );
     addAction( "next new strip" , mActionNextNewStripTab );
     mActions.append( mActionNextNewStripTab );
-    connect( mActionNextNewStripTab, SIGNAL(triggered(bool)), this, SLOT(slotNextNewStrip()) );
+    connect( mActionNextNewStripTab, SIGNAL(triggered(bool)), this, SIGNAL(showNextNewStrip()) );
 
     mActionGoFirst = new QAction( KIcon( "go-first" ), i18n( "Jump to &first Strip" ), this );
     mActions.append( mActionGoFirst );
@@ -185,107 +181,12 @@ void ComicApplet::init()
 
 ComicApplet::~ComicApplet()
 {
-    delete mFullViewWidget;
     delete mSavingDir;
 }
 
 QGraphicsWidget *ComicApplet::graphicsWidget()
 {
-    if ( !mMainWidget ) {
-        mMainWidget = new QGraphicsWidget( this );
-        mMainWidget->setMinimumSize( 150, 60 );
-        mMainWidget->setAcceptHoverEvents( true );
-        mMainWidget->installEventFilter( this );
-
-        QGraphicsLinearLayout *layout = new QGraphicsLinearLayout;
-        layout->setContentsMargins( 0, 0, 0, 0 );
-        layout->setSpacing( 0 );
-
-        mCentralLayout = new QGraphicsLinearLayout;
-        mCentralLayout->setOrientation( Qt::Vertical );
-        mCentralLayout->setContentsMargins( 0, 0, 0, 0 );
-        mCentralLayout->setSpacing( 0 );
-
-        mBottomLayout = new QGraphicsLinearLayout;
-        mBottomLayout->setContentsMargins( 0, 2, 0, 0 );
-        mBottomLayout->setSpacing( 0 );
-
-        mTabBar = new ComicTabBar( mMainWidget );
-        mTabBar->nativeWidget()->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
-        mTabBar->hide();
-        connect( mTabBar, SIGNAL(currentChanged(int)), this, SLOT(slotTabChanged(int)) );
-
-        mLabelTop = new ComicLabel( mMainWidget );
-        mLabelTop->setMinimumWidth( 0 );
-        mLabelTop->nativeWidget()->setWordWrap( false );
-        mLabelTop->nativeWidget()->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed ) );
-        mLabelTop->setAlignment( Qt::AlignCenter );
-        mLabelTop->hide();
-        mCentralLayout->addItem( mLabelTop );
-
-        mImageWidget = new ImageWidget( mMainWidget );
-        mImageWidget->setZValue( 0 );
-        mImageWidget->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
-        mCentralLayout->addItem( mImageWidget );
-
-        mLabelId = new ComicLabel( mMainWidget );
-        mLabelId->setMinimumWidth( 0 );
-        mLabelId->nativeWidget()->setWordWrap( false );
-        mLabelId->nativeWidget()->setCursor( Qt::PointingHandCursor );
-        mLabelId->nativeWidget()->setToolTip( i18n( "Jump to Strip ..." ) );
-        mLabelId->nativeWidget()->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed ) );
-        mLabelId->hide();
-        mBottomLayout->addItem( mLabelId );
-        mBottomLayout->addStretch( 1 );
-
-        mLabelUrl = new ComicLabel( mMainWidget );
-        mLabelUrl->setMinimumWidth( 0 );
-        mLabelUrl->nativeWidget()->setWordWrap( false );
-        if ( hasAuthorization( "LaunchApp" ) ) {
-            mLabelUrl->nativeWidget()->setCursor( Qt::PointingHandCursor );
-            mLabelUrl->nativeWidget()->setToolTip( i18n( "Visit the comic website" ) );
-        }
-        mLabelUrl->nativeWidget()->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed ) );
-        mLabelUrl->hide();
-        mBottomLayout->addItem( mLabelUrl );
-        mCentralLayout->addItem( mBottomLayout );
-
-        mLeftArrow = new ArrowWidget( mMainWidget );
-        mLeftArrow->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Expanding ) );
-        mLeftArrow->setCursor( Qt::PointingHandCursor );
-        mLeftArrow->hide();
-        connect( mLeftArrow, SIGNAL(clicked()), this, SLOT(slotPreviousDay()) );
-        layout->addItem( mLeftArrow );
-        layout->addItem( mCentralLayout );
-
-        mRightArrow = new ArrowWidget( mMainWidget );
-        mRightArrow->setDirection( Plasma::Right );
-        mRightArrow->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Expanding ) );
-        mRightArrow->setCursor( Qt::PointingHandCursor );
-        mRightArrow->hide();
-        connect( mRightArrow, SIGNAL(clicked()), this, SLOT(slotNextDay()) );
-        layout->addItem( mRightArrow );
-
-        mMainWidget->setLayout( layout );
-    }
-
-    return mMainWidget;
-}
-
-void ComicApplet::setTabBarVisible( bool isVisible )
-{
-    if ( !mCentralLayout ) {
-        return;
-    }
-
-    mTabBar->setVisible( isVisible );
-    if ( !mTabAdded && isVisible ) {
-        mCentralLayout->insertItem( 0, mTabBar );
-        mTabAdded = true;
-    } else if ( mTabAdded && !isVisible ) {
-        mCentralLayout->removeItem( mTabBar );
-        mTabAdded = false;
-    }
+    return mDeclarativeWidget;
 }
 
 void ComicApplet::dataUpdated( const QString &source, const Plasma::DataEngine::Data &data )
@@ -315,37 +216,17 @@ void ComicApplet::dataUpdated( const QString &source, const Plasma::DataEngine::
 
     setAssociatedApplicationUrls(mCurrent.websiteUrl());
 
-    // get the text at top
-    QString tempTop;
-    if ( mShowComicTitle ) {
-        tempTop = mCurrent.title();
-        tempTop += ( ( !mCurrent.stripTitle().isEmpty() && !mCurrent.title().isEmpty() ) ? " - " : "" ) + mCurrent.stripTitle();
-    }
-    
-    if ( mShowComicAuthor && !mCurrent.author().isEmpty() ) {
-        tempTop = ( !tempTop.isEmpty() ? mCurrent.author() + ": " + tempTop : mCurrent.author() );
-    }
-
     //looking at the last index, thus not mark it as new
     KConfigGroup cg = config();
     if (!mCurrent.hasNext() && mCheckNewComicStripsIntervall) {
-        const int index = mTabBar->currentIndex();
-        mTabBar->setTabHighlighted( index, false );
-        mActionNextNewStripTab->setEnabled( mTabBar->hasHighlightedTabs() );
+        setTabHighlighted( mCurrent.id(), false );
+        mActionNextNewStripTab->setEnabled( hasHighlightedTabs() );
     }
 
     //call the slot to check if the position needs to be saved
     slotStorePosition();
 
-    mLabelTop->setText( tempTop );
-    mImageWidget->setImage(mCurrent.image());
-    mImageWidget->setIsLeftToRight( data[ "isLeftToRight" ].toBool() );
-    mImageWidget->setIsTopToBottom( data[ "isTopToBottom" ].toBool() );
-    mLabelId->setText( mCurrent.currentReadable() );
-    mLabelUrl->setText( mCurrent.websiteUrl().host() );
-    mImageWidget->setScaled( !mCurrent.scaleComic() );
-
-    //disconnect if there is either no error, or an error that can not be fixed automatically 
+    //disconnect if there is either no error, or an error that can not be fixed automatically
     if ( !errorAutoFixable ) {
         mEngine->disconnectSource( source, this );
     }
@@ -363,37 +244,13 @@ void ComicApplet::dataUpdated( const QString &source, const Plasma::DataEngine::
     }
 
     updateView();
+
+    refreshComicData();
 }
 
 void ComicApplet::updateView()
 {
-    const bool tabsVisible = (mTabIdentifier.count() > 1);
-    setTabBarVisible( tabsVisible );
-    mLabelTop->setVisible( ( mShowComicAuthor || mShowComicTitle ) && !mLabelTop->text().isEmpty() );
-    mLabelId->setVisible( mShowComicIdentifier && !mLabelId->text().isEmpty() );
-    mLabelUrl->setVisible( mShowComicUrl && !mLabelUrl->text().isEmpty() );
-    const int spacing = ( mLabelTop->isVisible() ? 2 : 0 );
-    const int id = ( tabsVisible ? 1 : 0 );
-    mCentralLayout->setItemSpacing( id, spacing );
-    if ( mLabelId->isVisible() || mLabelUrl->isVisible() ) {
-        mBottomLayout->setContentsMargins( 0, 2, 0, 0 );
-    } else {
-        mBottomLayout->setContentsMargins( 0, 0, 0, 0 );
-    }
-
-    updateButtons();
     updateContextMenu();
-
-    Plasma::ToolTipContent toolTipData;
-    if ( !mCurrent.additionalText().isEmpty() ) {
-        toolTipData = Plasma::ToolTipContent( mCurrent.additionalText(), QString() );
-        toolTipData.setAutohide( false );
-    }
-    Plasma::ToolTipManager::self()->setContent( mMainWidget, toolTipData );
-
-    if (mCurrent.hasImage()) {
-        QTimer::singleShot( 1, this, SLOT(updateSize()) );//HACK
-    }
 }
 
 void ComicApplet::createConfigurationInterface( KConfigDialog *parent )
@@ -406,7 +263,6 @@ void ComicApplet::createConfigurationInterface( KConfigDialog *parent )
     mConfigWidget->setShowErrorPicture( mShowErrorPicture );
     mConfigWidget->setArrowsOnHover( mArrowsOnHover );
     mConfigWidget->setMiddleClick( mMiddleClick );
-    mConfigWidget->setTabView( mTabView - 1);//-1 because counting starts at 0, yet we use flags that start at 1
     mConfigWidget->setCheckNewComicStripsIntervall( mCheckNewComicStripsIntervall );
 
     //not storing this value, since other applets might have changed it inbetween
@@ -421,7 +277,6 @@ void ComicApplet::createConfigurationInterface( KConfigDialog *parent )
     parent->addPage( mConfigWidget->appearanceSettings, i18n( "Appearance" ), "image" );
     parent->addPage( mConfigWidget->advancedSettings, i18n( "Advanced" ), "system-run" );
 
-    connect( mConfigWidget, SIGNAL(maxSizeClicked()), this, SLOT(slotShowMaxSize()) );
     connect( parent, SIGNAL(applyClicked()), this, SLOT(applyConfig()) );
     connect( parent, SIGNAL(okClicked()), this, SLOT(applyConfig()) );
     connect(mConfigWidget, SIGNAL(enableApply()), parent, SLOT(settingsModified()));
@@ -429,14 +284,13 @@ void ComicApplet::createConfigurationInterface( KConfigDialog *parent )
 
 void ComicApplet::applyConfig()
 {
-    mShowComicUrl = mConfigWidget->showComicUrl();
-    mShowComicAuthor = mConfigWidget->showComicAuthor();
-    mShowComicTitle = mConfigWidget->showComicTitle();
-    mShowComicIdentifier = mConfigWidget->showComicIdentifier();
-    mShowErrorPicture = mConfigWidget->showErrorPicture();
-    mArrowsOnHover = mConfigWidget->arrowsOnHover();
-    mMiddleClick = mConfigWidget->middleClick();
-    mTabView = mConfigWidget->tabView() + 1;//+1 because counting starts at 0, yet we use flags that start at 1
+    setShowComicUrl(mConfigWidget->showComicUrl());
+    setShowComicAuthor(mConfigWidget->showComicAuthor());
+    setShowComicTitle(mConfigWidget->showComicTitle());
+    setShowComicIdentifier(mConfigWidget->showComicIdentifier());
+    setShowErrorPicture(mConfigWidget->showErrorPicture());
+    setArrowsOnHover(mConfigWidget->arrowsOnHover());
+    setMiddleClick(mConfigWidget->middleClick());
     mCheckNewComicStripsIntervall = mConfigWidget->checkNewComicStripsIntervall();
 
     //not storing this value, since other applets might have changed it inbetween
@@ -453,7 +307,6 @@ void ComicApplet::applyConfig()
 
     updateUsedComics();
     saveConfig();
-    buttonBar();
 
     //make sure that tabs etc. are displayed even if the comic strip in the first tab does not work
     updateView();
@@ -480,7 +333,8 @@ void ComicApplet::changeComic( bool differentComic )
 void ComicApplet::updateUsedComics()
 {
     const QString oldIdentifier = mCurrent.id();
-    mTabBar->removeAllTabs();
+
+    mActiveComicModel.clear();
     mTabIdentifier.clear();
     mCurrent = ComicData();
 
@@ -501,21 +355,14 @@ void ComicApplet::updateUsedComics()
                 mCurrent.setTitle(title);
             }
 
-            QIcon icon;
-            QString name;
+            const QString name = data.data().toString();
             const QString identifier = data.data( Qt::UserRole ).toString();
-            if ( mTabView & ShowText ) {
-                name = data.data().toString();
-            }
-            if ( mTabView & ShowIcon ) {
-                icon = data.data( Qt::DecorationRole ).value<QIcon>();
-            }
-
-            mTabBar->addTab( icon, name );
-
+            const QString iconPath = data.data( Qt::DecorationRole ).value<QIcon>().name();
             //found a newer strip last time, which was not visited
             if ( mCheckNewComicStripsIntervall && !cg.readEntry( "lastStripVisited_" + identifier, true ) ) {
-                mTabBar->setTabHighlighted( tab, true );
+                mActiveComicModel.addComic(identifier, name, iconPath, true);
+            } else {
+                mActiveComicModel.addComic(identifier, name, iconPath);
             }
 
             mTabIdentifier << identifier;
@@ -524,7 +371,7 @@ void ComicApplet::updateUsedComics()
     }
 
     mActionNextNewStripTab->setVisible( mCheckNewComicStripsIntervall );
-    mActionNextNewStripTab->setEnabled( mTabBar->hasHighlightedTabs() );
+    mActionNextNewStripTab->setEnabled( hasHighlightedTabs() );
 
     delete mCheckNewStrips;
     mCheckNewStrips = 0;
@@ -532,17 +379,15 @@ void ComicApplet::updateUsedComics()
         mCheckNewStrips = new CheckNewStrips( mTabIdentifier, mEngine, mCheckNewComicStripsIntervall, this );
         connect( mCheckNewStrips, SIGNAL(lastStrip(int,QString,QString)), this, SLOT(slotFoundLastStrip(int,QString,QString)) );
     }
+
+    emit comicModelChanged();
 }
 
-void ComicApplet::slotTabChanged( int newIndex )
+void ComicApplet::slotTabChanged(const QString &identifier)
 {
-    if ( newIndex >= mTabIdentifier.count() ) {
-        return;
-    }
-
-    bool differentComic = (mCurrent.id() != mTabIdentifier.at(newIndex));
+    bool differentComic = (mCurrent.id() != identifier);
     mCurrent = ComicData();
-    mCurrent.init(mTabIdentifier.at(newIndex), config());
+    mCurrent.init(identifier, config());
     changeComic( differentComic );
 }
 
@@ -577,8 +422,6 @@ void ComicApplet::configChanged()
     mMaxSize = cg.readEntry( "maxSize", tempMaxSize );
     mLastSize = mMaxSize;
 
-    mTabView = cg.readEntry( "tabView", ShowText | ShowIcon );
-
     globalComicUpdater->load();
 }
 
@@ -594,7 +437,6 @@ void ComicApplet::saveConfig()
     cg.writeEntry( "arrowsOnHover", mArrowsOnHover );
     cg.writeEntry( "middleClick", mMiddleClick );
     cg.writeEntry( "tabIdentifier", mTabIdentifier );
-    cg.writeEntry( "tabView", mTabView );
     cg.writeEntry( "checkNewComicStripsIntervall", mCheckNewComicStripsIntervall );
 
     globalComicUpdater->save();
@@ -625,23 +467,11 @@ void ComicApplet::slotFoundLastStrip( int index, const QString &identifier, cons
     KConfigGroup cg = config();
     if ( suffix != cg.readEntry( "lastStrip_" + identifier, QString() ) ) {
         kDebug() << identifier << "has a newer strip.";
-        mTabBar->setTabHighlighted( index, true );
+        setTabHighlighted( identifier, true );
         cg.writeEntry( "lastStripVisited_" + identifier, false );
     }
 
-    mActionNextNewStripTab->setEnabled( mTabBar->hasHighlightedTabs() );
-}
-
-void ComicApplet::slotReload()
-{
-    int index = mTabBar->currentIndex();
-    int newIndex = ( index + 1 ) % mTabBar->count();
-
-    if ( index == newIndex ) {
-        changeComic( false );
-    } else {
-        mTabBar->setCurrentIndex( newIndex );
-    }
+    mActionNextNewStripTab->setEnabled( hasHighlightedTabs() );
 }
 
 void ComicApplet::slotGoJump()
@@ -652,43 +482,11 @@ void ComicApplet::slotGoJump()
     selector->select(mCurrent);
 }
 
-void ComicApplet::slotNextNewStrip()
-{
-    const int index = mTabBar->currentIndex();
-    mTabBar->setCurrentIndex( mTabBar->nextHighlightedTab( index ) );
-}
-
 void ComicApplet::slotStorePosition()
 {
     mCurrent.storePosition(mActionStorePosition->isChecked());
 }
 
-void ComicApplet::slotSizeChanged()
-{
-    // if the applet was resized manually by the user
-    if ( isInPanel() ) {
-        if ( mMainWidget->size() != mLastSize ) {
-            mMaxSize = mMainWidget->size();
-            updateSize();
-
-            KConfigGroup cg = config();
-            cg.writeEntry( "maxSize", mMaxSize );
-        }
-    } else if ( this->geometry().size() != mLastSize ) {
-        mMaxSize = this->geometry().size();
-        updateSize();
-
-        KConfigGroup cg = config();
-        cg.writeEntry( "maxSize", mMaxSize );
-    }
-}
-
-void ComicApplet::slotShowMaxSize()
-{
-    resize( mMaxSize );
-    emit appletTransformedItself();
-}
-
 void ComicApplet::slotShop()
 {
     KRun::runUrl(mCurrent.shopUrl(), "text/html", 0);
@@ -699,33 +497,6 @@ bool ComicApplet::isInPanel() const
     return ( this->geometry().width() < 70 ) || ( this->geometry().height() < 50 );
 }
 
-void ComicApplet::updateSize()
-{
-    if ( configurationRequired() ) {
-        return;
-    }
-
-    QSizeF notAvailableSize;
-    if ( isInPanel() ) {
-        notAvailableSize =  mMainWidget->geometry().size() - mImageWidget->size();
-    } else {
-        notAvailableSize =  this->geometry().size() - mImageWidget->size();
-    }
-    QSizeF availableSize = mMaxSize - notAvailableSize;
-    mImageWidget->setAvailableSize( availableSize );
-    mLastSize = mImageWidget->preferredSize() + notAvailableSize;
-
-    if ( isInPanel() ) {
-        mMainWidget->resize( mLastSize );
-    } else {
-        resize( mLastSize );
-        mLastSize = this->size();//NOTE the applet won't be smaller than the minimum size of the MainWidget, thus the result of the resize might not correspond with mLastSize
-        emit sizeHintChanged( Qt::PreferredSize );
-        emit appletTransformedItself();
-    }
-    mImageWidget->update();
-}
-
 void ComicApplet::createComicBook()
 {
     ComicArchiveDialog *dialog = new ComicArchiveDialog(mCurrent.id(), mCurrent.title(), mCurrent.type(), mCurrent.current(),
@@ -813,20 +584,6 @@ void ComicApplet::updateComic( const QString &identifierSuffix )
     }
 }
 
-void ComicApplet::updateButtons()
-{
-    mShowNextButton = mCurrent.hasNext();
-    mShowPreviousButton = mCurrent.hasPrev();
-
-    mLeftArrow->setVisible(!mArrowsOnHover && mShowPreviousButton);
-    mRightArrow->setVisible(!mArrowsOnHover && mShowNextButton);
-
-    if (mButtonBar) {
-        mButtonBar->setNextEnabled(mShowNextButton);
-        mButtonBar->setPrevEnabled(mShowPreviousButton);
-    }
-}
-
 void ComicApplet::updateContextMenu()
 {
     mActionGoFirst->setVisible(mCurrent.hasFirst());
@@ -843,102 +600,205 @@ void ComicApplet::slotSaveComicAs()
     saver.save(mCurrent);
 }
 
-bool ComicApplet::eventFilter( QObject *receiver, QEvent *event )
+void ComicApplet::slotScaleToContent()
 {
-    if ( receiver != mMainWidget ) {
-        return Plasma::PopupApplet::eventFilter( receiver, event );
+    setShowActualSize(mActionScaleContent->isChecked());
+}
+
+//QML
+QObject *ComicApplet::comicsModel() 
+{
+    return &mActiveComicModel;
+}
+
+bool ComicApplet::showComicUrl() const
+{
+    return mShowComicUrl;
+}
+
+void ComicApplet::setShowComicUrl(bool show)
+{
+    if (show == mShowComicUrl)
+        return;
+
+    mShowComicUrl = show;
+
+    emit showComicUrlChanged();
+}
+
+bool ComicApplet::showComicAuthor() const
+{
+    return mShowComicAuthor;
+}
+
+void ComicApplet::setShowComicAuthor(bool show)
+{
+    if (show == mShowComicAuthor)
+        return;
+
+    mShowComicAuthor = show;
+
+    emit showComicAuthorChanged();
+}
+
+bool ComicApplet::showComicTitle() const
+{
+    return mShowComicTitle;
+}
+
+void ComicApplet::setShowComicTitle(bool show)
+{
+    if (show == mShowComicTitle)
+        return;
+
+    mShowComicTitle = show;
+
+    emit showComicTitleChanged();
+}
+
+bool ComicApplet::showComicIdentifier() const
+{
+    return mShowComicIdentifier;
+}
+
+void ComicApplet::setShowComicIdentifier(bool show)
+{
+    if (show == mShowComicIdentifier)
+        return;
+
+    mShowComicIdentifier = show;
+
+    emit showComicIdentifierChanged();
+}
+
+bool ComicApplet::showErrorPicture() const
+{
+    return mShowErrorPicture;
+}
+
+void ComicApplet::setShowErrorPicture(bool show)
+{
+    if (show == mShowErrorPicture)
+        return;
+
+    mShowErrorPicture = show;
+
+    emit showErrorPictureChanged();
+}
+
+bool ComicApplet::arrowsOnHover() const
+{
+    return mArrowsOnHover;
+}
+
+void ComicApplet::setArrowsOnHover(bool show)
+{
+    if (show == mArrowsOnHover)
+        return;
+
+    mArrowsOnHover = show;
+
+    emit arrowsOnHoverChanged();
+}
+
+bool ComicApplet::middleClick() const
+{
+    return mMiddleClick;
+}
+
+void ComicApplet::setMiddleClick(bool show)
+{
+    if (show == mMiddleClick)
+        return;
+
+    mMiddleClick = show;
+
+    emit middleClickChanged();
+}
+
+QVariantHash ComicApplet::comicData()
+{
+    return mComicData;
+}
+
+void ComicApplet::refreshComicData()
+{
+    mComicData["image"] = mCurrent.image();
+    mComicData["prev"] = mCurrent.prev();
+    mComicData["next"] = mCurrent.next();
+    mComicData["additionalText"] = mCurrent.additionalText();
+
+    mComicData["websiteUrl"] = mCurrent.websiteUrl().prettyUrl();
+    mComicData["websiteHost"] = mCurrent.websiteUrl().host();
+    mComicData["imageUrl"] = mCurrent.websiteUrl().prettyUrl();
+    mComicData["shopUrl"] = mCurrent.websiteUrl().prettyUrl();
+    mComicData["first"] = mCurrent.first();
+    mComicData["stripTitle"] = mCurrent.stripTitle();
+    mComicData["author"] = mCurrent.author();
+    mComicData["title"] = mCurrent.title();
+
+    mComicData["suffixType"] = "Date";
+    mComicData["current"] = mCurrent.current();
+    //mComicData["last"] = mCurrent.last();
+    mComicData["currentReadable"] = mCurrent.currentReadable();
+    mComicData["firstStripNum"] = mCurrent.firstStripNum();
+    mComicData["maxStripNum"] = mCurrent.maxStripNum();
+    mComicData["isLeftToRight"] = mCurrent.isLeftToRight();
+    mComicData["isTopToBottom"] = mCurrent.isTopToBottom();
+
+    emit comicDataChanged();
+}
+
+bool ComicApplet::showActualSize() const
+{
+    return mCurrent.scaleComic();
+}
+
+void ComicApplet::setShowActualSize(bool show)
+{
+    if (show == mCurrent.scaleComic())
+        return;
+
+    mCurrent.setScaleComic(show);
+
+    emit showActualSizeChanged();
+}
+//Endof QML
+void ComicApplet::setTabHighlighted(const QString &id, bool highlight)
+{
+    //Search for matching id
+    for (int index = 0; index < mActiveComicModel.rowCount(); ++index) {
+        QStandardItem * item = mActiveComicModel.item(index);
+
+        QString currentId = item->data(ActiveComicModel::ComicKeyRole).toString();
+        if (id == currentId){
+            if (highlight != item->data(ActiveComicModel::ComicHighlightRole).toBool()) {
+                item->setData(highlight, ActiveComicModel::ComicHighlightRole);
+                emit tabHighlightRequest(id, highlight);
+            }
+        }
     }
+}
 
-    switch (event->type()) {
-        case QEvent::GraphicsSceneHoverLeave:
-            if (mArrowsOnHover && mButtonBar) {
-                mButtonBar->hide();
-            }
-
-            break;
-        case QEvent::GraphicsSceneHoverEnter:
-            if (!configurationRequired() && mArrowsOnHover && mButtonBar) {
-                mButtonBar->show();
-            }
-
-            break;
-        case QEvent::GraphicsSceneMousePress:
-            {
-                QGraphicsSceneMouseEvent *e = static_cast<QGraphicsSceneMouseEvent *>( event );
-                if ( e->button() == Qt::LeftButton ) {
-                    if ( mLabelUrl->isUnderMouse() ) {
-                        if ( hasAuthorization( "LaunchApp" ) ) {
-                            // link clicked
-                            KRun::runUrl( mCurrent.websiteUrl(), "text/html", 0 );
-                            return true;
-                        }
-                    } else if ( mLabelId->isUnderMouse() ) {
-                        // identifierSuffix clicked clicked
-                        slotGoJump();
-                        return true;
-                    } else if ( mImageWidget->isUnderMouse() && ( mMainWidget->geometry().size() != mLastSize ) ) {
-                        // only update the size by clicking on the image-rect if the user manual resized the applet
-                        updateSize();
-                        return true;
-                    }
-                } else if ( ( e->button() == Qt::MidButton ) && mMiddleClick ) { // handle full view
-                    fullView();
-                    return true;
-                }
-            }
-            break;
-        case QEvent::GraphicsSceneResize:
-            if (mButtonBar) {
-                QPointF buttons((mMainWidget->size().width() - mButtonBar->size().width()) / 2,
-                                mMainWidget->contentsRect().bottom() - mButtonBar->size().height() - 5);
-                mButtonBar->setPos(buttons);
-            }
-
-            break;
-        default:
-            break;
+bool ComicApplet::hasHighlightedTabs()
+{
+    for (int i = 0; i < mActiveComicModel.rowCount(); ++i) {
+        if (isTabHighlighted(i)) {
+            return true;
+        }
     }
 
     return false;
 }
 
-void ComicApplet::slotScaleToContent()
+bool ComicApplet::isTabHighlighted(int index) const
 {
-    mCurrent.setScaleComic(mActionScaleContent->isChecked());
-    mImageWidget->setScaled(!mCurrent.scaleComic());
-
-    updateSize();
-}
-
-void ComicApplet::buttonBar()
-{
-    if (mArrowsOnHover) {
-        if (!mButtonBar) {
-            mButtonBar = new ButtonBar(mMainWidget);
-            connect(mButtonBar, SIGNAL(prevClicked()), this, SLOT(slotPreviousDay()));
-            connect(mButtonBar, SIGNAL(nextClicked()), this, SLOT(slotNextDay()));
-            connect(mButtonBar, SIGNAL(zoomClicked()), this, SLOT(fullView()));
-
-            // Set frame position
-            constraintsEvent(Plasma::SizeConstraint);
-        }
-    } else {
-        delete mButtonBar;
-        mButtonBar = 0;
-    }
-}
-
-void ComicApplet::fullView()
-{
-    if (!mFullViewWidget) {
-        mFullViewWidget = new FullViewWidget;
+    if (index < 0 || index >= mActiveComicModel.rowCount()) {
+        return false;
     }
 
-    if (!mFullViewWidget->isVisible()) {
-        mFullViewWidget->setImage(mCurrent.image());
-        mFullViewWidget->adaptPosition(mMainWidget->mapToScene( mMainWidget->pos()).toPoint(), containment()->screen());
-        mFullViewWidget->show();
-    }
-}
+    QStandardItem * item = mActiveComicModel.item(index);
 
+    return item->data(ActiveComicModel::ComicHighlightRole).toBool();
+}
 #include "comic.moc"
