@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2007 by Thomas Georgiou <TAGeorgiou@gmail.com>          *
- *                         Artur Duque de Souza <morpheuz@gmail.com>       *
+ *                         Artur Duque de Souza <asouza@kde.org>           *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -28,13 +28,16 @@
 
 #include <kmimetype.h>
 #include <KTemporaryFile>
-
+#include <KDE/KIO/MimetypeJob>
+#include <KDebug>
 
 #include <kio/global.h>
 #include <kio/job.h>
 
 #include <Plasma/Theme>
 #include <Plasma/Service>
+#include <Plasma/ServiceJob>
+#include <KNS3/DownloadDialog>
 
 #include <QApplication>
 #include <QClipboard>
@@ -46,39 +49,50 @@
 #include <QPainter>
 #include <QPaintEngine>
 #include <QSignalMapper>
+#include <QPropertyAnimation>
 
 
 Pastebin::Pastebin(QObject *parent, const QVariantList &args)
     : Plasma::Applet(parent, args),
-      m_textBackend(0), m_imageBackend(0), m_imagePrivacy(0),
-      m_historySize(3), m_signalMapper(new QSignalMapper()), m_paste(0),
-      m_topSeparator(0), m_bottomSeparator(0), m_servers(0), m_waiting(false)
+      m_signalMapper(new QSignalMapper()), m_paste(0),
+      m_topSeparator(0), m_bottomSeparator(0),
+      m_historySize(3), m_newStuffDialog(0)
 {
     setAcceptDrops(true);
     setHasConfigurationInterface(true);
     setAspectRatioMode(Plasma::ConstrainedSquare);
     setMinimumSize(16, 16);
     resize(150, 150);
-    timer = new QTimer(this);
-    connect(timer, SIGNAL(timeout()), this, SLOT(showErrors()));
+    m_timer = new QTimer(this);
+    connect(m_timer, SIGNAL(timeout()), this, SLOT(showErrors()));
 
-    connect(m_signalMapper, SIGNAL(mapped(const QString &)),
-             this, SLOT(copyToClipboard(const QString &)));
+    connect(m_signalMapper, SIGNAL(mapped(QString)),
+             this, SLOT(copyToClipboard(QString)));
+    connect(this, SIGNAL(activate()), this, SLOT(postClipboard()));
 
-    Plasma::DataEngine *engine = dataEngine("pastebin");
-    engine->connectSource("result", this);
-    engine->connectSource("error", this);
+    // connect to all sources of our 'share' dataengine
+    m_engine = dataEngine("org.kde.plasma.dataengine.share");
+    m_engine->connectAllSources(this);
+
+    // to detect when the mimetypes were added again after a refresh
+    connect(m_engine, SIGNAL(sourceAdded(QString)),
+            this, SLOT(sourceAdded(QString)));
+    connect(m_engine, SIGNAL(sourceRemoved(QString)),
+            this, SLOT(sourceRemoved(QString)));
 }
 
 Pastebin::~Pastebin()
 {
     delete m_topSeparator;
     delete m_bottomSeparator;
-    delete m_icon;
+    delete m_newStuffDialog;
+
+    // save history of URLs
     QString history;
-    for (int i = 0; i < m_actionHistory.size(); ++i) {
-        history.prepend(m_actionHistory.at(i)->text());
-        history.prepend("|");
+    const int numberOfActionHistory = m_actionHistory.size();
+    for (int i = 0; i < numberOfActionHistory; ++i) {
+        history.prepend(m_actionHistory.at(i)->toolTip());
+        history.prepend('|');
         delete m_actionHistory.at(i);
     }
 
@@ -88,23 +102,9 @@ Pastebin::~Pastebin()
 
 void Pastebin::init()
 {
-    KConfigGroup cg = config();
-
-    m_textBackend = cg.readEntry("TextBackend", "0").toInt();
-    m_imageBackend = cg.readEntry("ImageBackend", "0").toInt();
-    m_imagePrivacy = cg.readEntry("imagePrivacy", "0").toInt();
-    int historySize = cg.readEntry("HistorySize", "3").toInt();
-
-    QStringList history = cg.readEntry("History", "").split('|', QString::SkipEmptyParts);
-
-    for (int i = 0; i < history.size(); ++i) {
-        addToHistory(history.at(i));
-    }
-
-    setHistorySize(historySize);
+    configChanged();
     setActionState(Idle);
     setInteractionState(Waiting);
-    m_icon = new KIcon("edit-paste"); // TODO: make member (for caching)
 
     updateTheme();
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(updateTheme()));
@@ -112,33 +112,56 @@ void Pastebin::init()
     Plasma::ToolTipManager::self()->setContent(this, toolTipData);
 }
 
-void Pastebin::dataUpdated(const QString &sourceName, const Plasma::DataEngine::Data &data)
+void Pastebin::dataUpdated(const QString &source, const Plasma::DataEngine::Data &data)
 {
-    // if it's something not generated from this applet, ignore it
-    if (!m_waiting) {
-        return;
-    }
+    // update the options
+    if (source != "Mimetypes") {
+        const QString mimetype = data.value("Mimetypes").toString();
 
-    // initialization of data
-    if (data["result"].toString().isEmpty() && data["error"].toString().isEmpty()) {
-        return;
+        if (mimetype.startsWith("text/")) {
+            m_txtServers.insert(data.value("Name").toString(), source);
+        } else if (mimetype.startsWith("image/")) {
+            m_imgServers.insert(data.value("Name").toString(), source);
+        } else {
+            kDebug() << "Mimetype not supported by this applet";
+        }
     }
+}
 
-    m_waiting = false;
-    const QString msg(data[sourceName].toString());
-    if (sourceName == "result") {
-        const QString result(data["result"].toString());
-        showResults(msg);
-    } else {
-        showErrors();
+void Pastebin::sourceAdded(const QString &source)
+{
+    // update the options
+    if (source != "Mimetypes") {
+        const Plasma::DataEngine::Data data = m_engine->query(source);
+        const QString mimetype = data.value("Mimetypes").toString();
+
+        if (mimetype.startsWith("text/")) {
+            m_txtServers.insert(data.value("Name").toString(), source);
+        } else if (mimetype.startsWith("image/")) {
+            m_imgServers.insert(data.value("Name").toString(), source);
+        } else {
+            kDebug() << "Mimetype not supported by this applet";
+        }
+    }
+}
+
+void Pastebin::sourceRemoved(const QString &source)
+{
+    // update the options
+    if (source != "Mimetypes") {
+        QString key = m_txtServers.key(source);
+        m_txtServers.remove(key);
+
+        key = m_imgServers.key(source);
+        m_imgServers.remove(key);
     }
 }
 
 void Pastebin::setHistorySize(int max)
 {
-    if (max <= 0)
+    if (max <= 0) {
         max = 0;
-
+    }
     while (max < m_actionHistory.count()) {
         delete m_actionHistory.takeFirst();
     }
@@ -206,7 +229,7 @@ void Pastebin::setActionState(ActionState state)
             toolTipData.setImage(KIcon("dialog-cancel"));
             // Notification ...
             QTimer::singleShot(15000, this, SLOT(resetActionState()));
-            timer->stop();
+            m_timer->stop();
             break;
         case IdleSuccess:
             setBusy(false);
@@ -214,7 +237,7 @@ void Pastebin::setActionState(ActionState state)
             toolTipData.setImage(KIcon("dialog-ok"));
             // Notification ...
             QTimer::singleShot(15000, this, SLOT(resetActionState()));
-            timer->stop();
+            m_timer->stop();
             break;
         case Sending:
             setBusy(true);
@@ -232,12 +255,6 @@ void Pastebin::setActionState(ActionState state)
 
 void Pastebin::constraintsEvent(Plasma::Constraints constraints)
 {
-    if (constraints & (Plasma::FormFactorConstraint | Plasma::SizeConstraint)) {
-        int minSize = KGlobalSettings::smallestReadableFont().pointSize();
-        int dynSize = qMax((qreal)1.0, (qreal)(qMin(contentsRect().width(), contentsRect().height()) / 4));
-        //kDebug() << "Min : Dyn" << minSize << dynSize << qMax(minSize, dynSize);
-        m_font.setPointSize(qMax(minSize, dynSize));
-    }
     if (constraints & Plasma::FormFactorConstraint) {
         if (formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical ) {
             setPreferredSize(-1, -1);
@@ -328,11 +345,12 @@ void Pastebin::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *, con
         QPixmap iconok = KIcon("dialog-cancel").pixmap(QSize(iconsize, iconsize));
         paintPixmap(p, iconok, iconRect, pix_alpha);
     } else {
-        QPixmap iconpix = m_icon->pixmap(QSize(iconsize, iconsize));
-        paintPixmap(p, iconpix, iconRect, pix_alpha);
+        QPixmap iconpix = KIcon("edit-paste").pixmap(QSize(iconsize, iconsize));
+        if (!iconpix.isNull()) {
+            paintPixmap(p, iconpix, iconRect, pix_alpha);
+        }
     }
 
-
     // Draw background
     if (m_interactionState == DraggedOver) {
         m_fgColor.setAlphaF(m_alpha);
@@ -352,31 +370,6 @@ void Pastebin::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *, con
     qreal proportion = contentsRect.width() / contentsRect.height();
     qreal round_radius = 35.0;
     p->drawRoundedRect(contentsRect, round_radius / proportion, round_radius, Qt::RelativeSize);
-
-    QString hoverText = i18n("Drop!");
-    if (m_interactionState == DraggedOver) {
-        m_fgColor.setAlphaF(m_alpha * 0.8);
-    } else if (m_interactionState == Hovered) {
-        m_fgColor.setAlphaF(m_alpha * 0.6);
-    }
-    p->drawText(contentsRect, Qt::AlignCenter, hoverText);
-    p->setPen(m_fgColor);
-
-    if (m_actionState == IdleSuccess) {
-        QFont font = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
-        font.setPointSize(KGlobalSettings::smallestReadableFont().pointSize());
-        p->setFont(font);
-        p->drawText(contentsRect, Qt::AlignCenter, m_url);
-    } else if (m_actionState == IdleError) {
-        QFont font = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
-        font.setPointSize(KGlobalSettings::smallestReadableFont().pointSize());
-        font.setBold(true);
-        p->setFont(font);
-        p->drawText(contentsRect, Qt::AlignCenter, i18n("Error: Try Again"));
-    } else {
-        p->setFont(m_font);
-        p->drawText(contentsRect, Qt::AlignCenter, hoverText);
-    }
 }
 
 void Pastebin::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
@@ -397,31 +390,73 @@ void Pastebin::showOverlay(bool show)
         return;
     }
     m_fadeIn = show;
-    const int FadeInDuration = 400;
 
-    if (m_animId != -1) {
-        Plasma::Animator::self()->stopCustomAnimation(m_animId);
+    QPropertyAnimation *animation = m_animation.data();
+    if (!animation) {
+        animation = new QPropertyAnimation(this, "animationUpdate");
+        animation->setDuration(400);
+        animation->setStartValue(0.0);
+        animation->setEndValue(1.0);
+        animation->setEasingCurve(QEasingCurve::Linear);
+        m_animation = animation;
+    } else if (animation->state() == QAbstractAnimation::Running) {
+        animation->pause();
     }
-    m_animId = Plasma::Animator::self()->customAnimation(40 / (1000 / FadeInDuration), FadeInDuration,
-                                                      Plasma::Animator::EaseOutCurve, this,
-                                                      "animationUpdate");
+
+    if (m_fadeIn) {
+        animation->setDirection(QAbstractAnimation::Forward);
+        animation->start(QAbstractAnimation::KeepWhenStopped);
+    } else {
+        animation->setDirection(QAbstractAnimation::Backward);
+        animation->start(QAbstractAnimation::DeleteWhenStopped);
+    }
+}
+
+qreal Pastebin::animationValue() const
+{
+    return m_alpha;
 }
 
 void Pastebin::animationUpdate(qreal progress)
 {
-    if (progress == 1) {
-        m_animId = -1;
-    }
-    if (!m_fadeIn) {
-        qreal new_alpha = m_fadeIn ? progress : 1 - progress;
-        m_alpha = qMin(new_alpha, m_alpha);
-    } else {
-        m_alpha = m_fadeIn ? progress : 1 - progress;
-    }
-    m_alpha = qMax(qreal(0.0), m_alpha);
+    m_alpha = progress;
     update();
 }
 
+void Pastebin::getNewStuff()
+{
+    if (!m_newStuffDialog) {
+        QString ghns("pastebin.knsrc");
+        m_newStuffDialog = new KNS3::DownloadDialog( ghns );
+        connect(m_newStuffDialog, SIGNAL(accepted()),
+                this, SLOT(newStuffFinished()));
+    }
+    m_newStuffDialog->show();
+}
+
+void Pastebin::newStuffFinished()
+{
+    if ( m_newStuffDialog->changedEntries().count() ) {
+        // refresh the options of config dialog
+        refreshConfigDialog();
+
+        // setup the config dialog to last options
+        KConfigGroup cg = config();
+        uiConfig.textServer->setCurrentItem(cg.readEntry("TextProvider", ""));
+        uiConfig.imageServer->setCurrentItem(cg.readEntry("ImageProvider", ""));
+    }
+}
+
+void Pastebin::refreshConfigDialog()
+{
+    // setup text
+    uiConfig.textServer->clear();
+    uiConfig.textServer->addItems(m_txtServers.keys());
+
+    // setup image
+    uiConfig.imageServer->clear();
+    uiConfig.imageServer->addItems(m_imgServers.keys());
+}
 
 void Pastebin::createConfigurationInterface(KConfigDialog *parent)
 {
@@ -431,149 +466,54 @@ void Pastebin::createConfigurationInterface(KConfigDialog *parent)
     uiConfig.setupUi(general);
 
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
-    connect(parent, SIGNAL(cancelClicked()), this, SLOT(closeServerDialog()));
     parent->addPage(general, i18n("General"), Applet::icon());
 
-    uiConfig.textServer->setCurrentIndex(m_textBackend);
-    uiConfig.imageServer->setCurrentIndex(m_imageBackend);
+    uiConfig.ghnsButton->setIcon(KIcon("get-hot-new-stuff"));
+    connect(uiConfig.ghnsButton, SIGNAL(clicked()), this, SLOT(getNewStuff()));
+
+    refreshConfigDialog();
+    uiConfig.textServer->setCurrentItem(cg.readEntry("TextProvider", m_txtServers.keys().at(0)));
+    uiConfig.imageServer->setCurrentItem(cg.readEntry("ImageProvider", m_imgServers.keys().at(0)));
     uiConfig.historySize->setValue(m_historySize);
-    uiConfig.imagePrivacy->setCurrentIndex(m_imagePrivacy);
-
-    uiConfig.editTextServer->setIcon(KIcon("document-edit"));
-    uiConfig.editImageServer->setIcon(KIcon("document-edit"));
-
-    connect(uiConfig.editTextServer, SIGNAL(clicked()), this, SLOT(editTextServer()));
-    connect(uiConfig.editImageServer, SIGNAL(clicked()), this, SLOT(editImageServer()));
-}
-
-void Pastebin::editTextServer()
-{
-    KConfigGroup cg = config();
-
-    // avoid memory leaks as its not modal and the user can click again
-    // on the edit button
-    delete m_servers;
-
-    m_servers = new QWidget();
-    uiServers.setupUi(m_servers);
-    m_servers->show();
-    connect(uiServers.buttonBox, SIGNAL(accepted()), this, SLOT(saveTextServer()));
-    connect(uiServers.buttonBox, SIGNAL(rejected()), this, SLOT(closeServerDialog()));
-    QString serverAddress;
-    switch(uiConfig.textServer->currentIndex()) {
-        case Pastebin::PASTEBINCA:
-            serverAddress = cg.readEntry("pastebinca", "http://pastebin.ca");
-            break;
-
-        case Pastebin::PASTEBINCOM:
-           serverAddress = cg.readEntry("pastebincom", "http://pastebin.com");
-           break;
-    }
-
-    uiServers.server->setText(serverAddress);
-}
-
-void Pastebin::saveTextServer()
-{
-    KConfigGroup cg = config();
-
-    switch(uiConfig.textServer->currentIndex()) {
-        case Pastebin::PASTEBINCA:
-            cg.writeEntry("pastebinca", uiServers.server->text());
-            break;
-
-        case Pastebin::PASTEBINCOM:
-            cg.writeEntry("pastebincom", uiServers.server->text());
-            break;
-    }
-
-    closeServerDialog();
-}
-
-void Pastebin::closeServerDialog()
-{
-    delete m_servers;
-    m_servers = 0;
-}
-
-void Pastebin::editImageServer()
-{
-    KConfigGroup cg = config();
-
-    // avoid memory leaks as its not modal and the user can click again
-    // on the edit button
-    delete m_servers;
-
-    m_servers = new QWidget();
-    uiServers.setupUi(m_servers);
-    m_servers->show();
-    connect(uiServers.buttonBox, SIGNAL(accepted()), this, SLOT(saveImageServer()));
-    connect( uiServers.buttonBox, SIGNAL( rejected() ), this, SLOT( closeServerDialog() ) );
-    QString serverAddress;
-    switch(uiConfig.imageServer->currentIndex()) {
-        case Pastebin::IMAGEBINCA:
-            serverAddress = cg.readEntry("imagebinca", "http://imagebin.ca");
-            break;
-
-        case Pastebin::IMAGESHACK:
-            serverAddress = cg.readEntry("imageshack", "http://imageshack.us");
-            break;
-
-        case Pastebin::SIMPLESTIMAGEHOSTING:
-            serverAddress = cg.readEntry("simplestimagehosting",
-                                         "http://api.simplest-image-hosting.net");
-            break;
-    }
-
-    uiServers.server->setText(serverAddress);
-}
-
-void Pastebin::saveImageServer()
-{
-    KConfigGroup cg = config();
-
-    switch(uiConfig.imageServer->currentIndex()) {
-        case Pastebin::IMAGEBINCA:
-            cg.writeEntry("imagebinca", uiServers.server->text());
-            break;
-
-        case Pastebin::IMAGESHACK:
-            cg.writeEntry("imageshack", uiServers.server->text());
-            break;
-
-        case Pastebin::SIMPLESTIMAGEHOSTING:
-            cg.writeEntry("simplestimagehosting", uiServers.server->text());
-            break;
-    }
-    closeServerDialog();
+    
+    connect(uiConfig.textServer , SIGNAL(currentIndexChanged(int)) , parent, SLOT(settingsModified()));
+    connect(uiConfig.imageServer , SIGNAL(currentIndexChanged(int)) , parent, SLOT(settingsModified()));
+    connect(uiConfig.historySize , SIGNAL(valueChanged(int)) , parent, SLOT(settingsModified()));
 }
 
 void Pastebin::configAccepted()
 {
     KConfigGroup cg = config();
-    m_textBackend = uiConfig.textServer->currentIndex();
-    m_imageBackend = uiConfig.imageServer->currentIndex();
-    m_imagePrivacy = uiConfig.imagePrivacy->currentIndex();
-
     int historySize = uiConfig.historySize->value();
     setHistorySize(historySize);
 
-    cg.writeEntry("TextBackend", m_textBackend);
-    cg.writeEntry("ImageBackend", m_imageBackend);
-    cg.writeEntry("imagePrivacy", m_imagePrivacy);
+    cg.writeEntry("TextProvider", uiConfig.textServer->currentText());
+    cg.writeEntry("ImageProvider", uiConfig.imageServer->currentText());
     cg.writeEntry("HistorySize", historySize);
 
-    closeServerDialog();
     emit configNeedsSaving();
 }
 
+void Pastebin::configChanged()
+{
+    KConfigGroup cg = config();
+    int historySize = cg.readEntry("HistorySize", "3").toInt();
+    QStringList history = cg.readEntry("History", "").split('|', QString::SkipEmptyParts);
+
+    m_actionHistory.clear();
+    const int numberOfItems = history.size();
+    for (int i = 0; i < numberOfItems; ++i) {
+        addToHistory(history.at(i));
+    }
+    setHistorySize(historySize);
+}
+
 void Pastebin::showResults(const QString &url)
 {
-    timer->stop();
+    m_timer->stop();
     m_url = url;
     setActionState(IdleSuccess);
     copyToClipboard(url);
-
     addToHistory(url);
 }
 
@@ -581,12 +521,13 @@ void Pastebin::copyToClipboard(const QString &url)
 {
     QApplication::clipboard()->setText(url, lastMode);
     kDebug() << "Copying:" << url;
+    m_oldUrl = url;
     QPixmap pix = KIcon("edit-paste").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
 
     KNotification *notify = new KNotification("urlcopied");
     notify->setComponentData(KComponentData("plasma_pastebin"));
     notify->setText(i18nc("Notification when the pastebin applet has copied the URL to the clipboard",
-                          "%1 has been copied to your clipboard", url));
+                          "The URL for your paste has been copied to the clipboard"));
     notify->setPixmap(pix);
     notify->setActions(QStringList(i18n("Open browser")));
     connect(notify, SIGNAL(action1Activated()), this, SLOT(openLink()));
@@ -598,9 +539,13 @@ void Pastebin::showErrors()
     setActionState(IdleError);
 }
 
-void Pastebin::openLink()
+void Pastebin::openLink(bool old)
 {
-    KToolInvocation::invokeBrowser(m_url);
+    if(old) {
+      KToolInvocation::invokeBrowser(m_oldUrl);
+    } else {
+      KToolInvocation::invokeBrowser(m_url);
+    }
 }
 
 void Pastebin::resetActionState()
@@ -614,29 +559,12 @@ void Pastebin::mousePressEvent(QGraphicsSceneMouseEvent *event)
     if (m_url.isEmpty() || event->button() != Qt::LeftButton) {
         Plasma::Applet::mousePressEvent(event);
     } else {
-        openLink();
+        openLink(false);
     }
     if (event->button() == Qt::MidButton) {
         if (m_actionState == Idle) {
             // paste clipboard content
-#ifdef Q_WS_WIN
-            // Same as for D'n'D, Windows doesn't pass any actual image data when pasting
-            // image files. Though, it does provide us with those files' Urls. Since posting
-            // multiple images isn't yet implemented - we'll use first Url from list
-            lastMode = QClipboard::Clipboard;
-            QImage image;
-            QString imageFileName;
-            if (QApplication::clipboard()->mimeData()->hasUrls()) {
-                imageFileName = QApplication::clipboard()->mimeData()->urls().at(0).toLocalFile();
-                image.load(imageFileName);
-                postContent(imageFileName, image);
-            } else {
-                postContent(QApplication::clipboard()->mimeData()->text(), image);
-            };
-#else
-            lastMode = QApplication::clipboard()->supportsSelection() ? QClipboard::Selection : QClipboard::Clipboard;
-            postContent(QApplication::clipboard()->text(lastMode), QApplication::clipboard()->image(lastMode));
-#endif //Q_WS_WIN
+            postClipboard(true);
         } else {
             // Now releasing the middlebutton click copies to clipboard
             event->accept();
@@ -749,6 +677,11 @@ QList<QAction*> Pastebin::contextualActions()
 }
 
 void Pastebin::postClipboard()
+{
+    postClipboard(false);
+}
+
+void Pastebin::postClipboard(bool preferSelection)
 {
     lastMode = QClipboard::Clipboard;
 #ifdef Q_WS_WIN
@@ -765,96 +698,94 @@ void Pastebin::postClipboard()
         postContent(QApplication::clipboard()->mimeData()->text(), image);
     };
 #else
-    postContent(QApplication::clipboard()->text(), QApplication::clipboard()->image());
+    if (preferSelection) {
+        lastMode = QApplication::clipboard()->supportsSelection() ? QClipboard::Selection : QClipboard::Clipboard;
+        postContent(QApplication::clipboard()->text(lastMode), QApplication::clipboard()->image(lastMode));
+    } else {
+        postContent(QApplication::clipboard()->text(), QApplication::clipboard()->image());
+    }
 #endif //Q_WS_WIN
 }
 
-void Pastebin::postContent(QString text, QImage imageData)
+void Pastebin::postContent(QString text, const QImage& imageData)
 {
+    QString sourceName;
     KUrl testPath(text);
-    bool validPath = QFile::exists(testPath.toLocalFile());
-    bool image = false;
+    bool validPath = true;
+
+    // use KIO to check if the file exists using mimetype job
+    KIO::MimetypeJob *mjob = KIO::mimetype(testPath);
+    if (!mjob->exec()) {
+        // it's not a file - usually this happens when we are
+        // just sharing plain text or image
+        validPath = false;
+    }
+
+    KConfigGroup cg = config();
+    // This is needed to provide smooth transition between old config and new one
+    const QString txtProvider = cg.readEntry("TextProvider", m_txtServers.keys().at(0));
+    const QString imgProvider = cg.readEntry("ImageProvider", m_imgServers.keys().at(0));
 
     if (validPath) {
         KMimeType::Ptr type = KMimeType::findByPath(testPath.path());
 
         if (type->name().indexOf("image/") != -1) {
-            image = true;
+            // its image
+            sourceName = m_imgServers.value(imgProvider);
+        } else {
+            // its text
+            sourceName = m_txtServers.value(txtProvider);
         }
+    } else if (imageData.isNull()) {
+        sourceName = m_txtServers.value(txtProvider);
     } else {
-        if (!imageData.isNull()) {
-            image = true;
+        sourceName = m_imgServers.value(imgProvider);
 
-            KTemporaryFile tempFile;
-            if (tempFile.open()) {
-                tempFile.setAutoRemove(false);
+        KTemporaryFile tempFile;
+        tempFile.setSuffix(".png");
+        if (tempFile.open()) {
+            tempFile.setAutoRemove(false);
 
-                QDataStream stream(&tempFile);
-                QByteArray data;
-                QBuffer buffer(&data);
+            imageData.save(&tempFile, "PNG");
+            tempFile.close();
 
-                buffer.open(QIODevice::ReadWrite);
-                imageData.save(&buffer, "JPEG");
-                stream.writeRawData(data, data.size());
-                tempFile.close();
-
-                text = tempFile.fileName();
-            } else {
-                setActionState(IdleError);
-                return;
-            }
+            text = tempFile.fileName();
+        } else {
+            setActionState(IdleError);
+            return;
         }
     }
 
-    KConfigGroup cg = config();
+    kDebug() << "Is valid path: " << validPath;
+    kDebug() << "Provider used: " << sourceName;
 
-    if (!image) {
-        Plasma::Service *service = dataEngine("pastebin")->serviceForSource("");
-        KConfigGroup ops = service->operationDescription("text");
-
-        switch(m_textBackend) {
-            case Pastebin::PASTEBINCA:
-                ops.writeEntry("server", cg.readEntry("pastebinca", "http://pastebin.ca"));
-                break;
-
-            case Pastebin::PASTEBINCOM:
-                ops.writeEntry("server", cg.readEntry("pastebincom", "http://pastebin.com"));
-                break;
-        }
-
-        ops.writeEntry("fileName", text);
-        ops.writeEntry("backend", m_textBackend);
-        service->startOperationCall(ops);
-    } else {
-        Plasma::Service *service = dataEngine("pastebin")->serviceForSource("");
-        KConfigGroup ops = service->operationDescription("image");
-
-        switch(m_imageBackend) {
-            case Pastebin::IMAGEBINCA:
-                ops.writeEntry("server", cg.readEntry("imagebinca", "http://imagebin.ca"));
-                break;
-
-            case Pastebin::IMAGESHACK:
-                ops.writeEntry("server", cg.readEntry("imageshack", "http://imageshack.us"));
-                break;
-
-            case Pastebin::SIMPLESTIMAGEHOSTING:
-                ops.writeEntry("server", cg.readEntry("simplestimagehosting",
-                                                      "http://api.simplest-image-hosting.net"));
-                break;
-        }
-
-        ops.writeEntry("fileName", text);
-        ops.writeEntry("privacy", m_imagePrivacy);
-        ops.writeEntry("backend", m_imageBackend);
-
-        service->startOperationCall(ops);
+    if (sourceName.isEmpty()) {
+        // no provider was configured
+        showErrors();
+        return;
     }
 
+    m_postingService = m_engine->serviceForSource(sourceName);
+    KConfigGroup ops = m_postingService->operationDescription("share");
+    ops.writeEntry("content", text);
+
+    Plasma::ServiceJob *job = m_postingService->startOperationCall(ops);
+    connect(job, SIGNAL(finished(KJob*)), this, SLOT(postingFinished(KJob*)));
+
     setActionState(Sending);
-    timer->start(20000);
-    m_waiting = true;
+    m_timer->start(20000);
+}
+
+void Pastebin::postingFinished(KJob *job)
+{
+    Plasma::ServiceJob *sjob = static_cast<Plasma::ServiceJob*>(job);
+    if (sjob->error()) {
+        showErrors();
+    } else {
+        showResults(sjob->result().toString());
+    }
 }
 
 
+
 #include "pastebin.moc"
